<?php

namespace Drupal\drupalorg\Controller;

use Drupal\Component\Serialization\Json;
use Drupal\Core\Controller\ControllerBase;
use Drupal\Core\Queue\QueueFactory;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Symfony\Component\HttpFoundation\JsonResponse;
use Symfony\Component\HttpFoundation\Request;

/**
 * Controller to manage system webhooks.
 */
class WebhooksController extends ControllerBase {

  /**
   * Queue factory service.
   *
   * @var \Drupal\Core\Queue\QueueFactory
   */
  protected QueueFactory $queueFactory;

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('queue')
    );
  }

  /**
   * Construct method.
   *
   * @param \Drupal\Core\Queue\QueueFactory $queue_factory
   *   Queue factory.
   */
  public function __construct(QueueFactory $queue_factory) {
    $this->queueFactory = $queue_factory;
  }

  /**
   * Webhook callback.
   *
   * @param \Symfony\Component\HttpFoundation\Request $request
   *   The current request.
   *
   * @return \Symfony\Component\HttpFoundation\JsonResponse
   *   Response object.
   */
  public function projectWebhook(Request $request): JsonResponse {
    $result = [
      'status' => 'error',
      'message' => $this->t('Unknown error'),
    ];

    // Check headers and token.
    $gitlab_token = $this->config('drupalorg.gitlab_settings')->get('token');
    if (
      !empty($gitlab_token) &&
      ($request->server->get('HTTP_X_GITLAB_TOKEN') === $gitlab_token)
    ) {
      $data = Json::decode($request->getContent());
      if (empty($data['event_name'])) {
        $result['message'] = $this->t('Webhook payload does not contain the event name');
      }
      elseif ($data['event_name'] == 'project_update') {
        $this->queueFactory->get('drupalorg_project_activity_webhook_queue_worker')->createItem([
          'event_name' => 'project_update',
          'project_id' => $data['project_id'] ?? NULL,
        ]);
        $result['message'] = $this->t('Event was queued successfully');
        $result['status'] = 'success';
      }
      else {
        $result['message'] = $this->t('Event not covered by code');
        $result['status'] = 'warning';
      }
    }
    else {
      $result['message'] = $this->t('Invalid token');
    }

    return new JsonResponse($result);
  }

  /**
   * Webhook callback for activities related to contributions.
   *
   * @param \Symfony\Component\HttpFoundation\Request $request
   *   The current request.
   *
   * @return \Symfony\Component\HttpFoundation\JsonResponse
   *   Response object.
   */
  public function contributionActivityWebhook(Request $request): JsonResponse {
    $result = [
      'status' => 'error',
      'message' => $this->t('Unknown error'),
    ];

    // Check headers and token.
    $gitlab_token = $this->config('drupalorg.gitlab_settings')->get('token');
    if (
      !empty($gitlab_token) &&
      ($request->server->get('HTTP_X_GITLAB_TOKEN') === $gitlab_token)
    ) {
      $result = $this->contributionActivityWebhookFromGitLab($request);
    }
    elseif (
      !empty($gitlab_token) &&
      ($request->headers->get('Drupalorg-Webhook-Token') === $gitlab_token)
    ) {
      // Depends on https://www.drupal.org/project/drupalorg/issues/3327584
      $result = $this->contributionActivityWebhookFromDrupal($request);
    }
    else {
      $result['message'] = $this->t('Invalid token');
    }

    return new JsonResponse($result);
  }

  /**
   * Process the webhook request from gitlab.
   *
   * @param \Symfony\Component\HttpFoundation\Request $request
   *   Request object.
   *
   * @return array
   *   Result to return.
   */
  protected function contributionActivityWebhookFromGitLab(Request $request): array {
    $result = [
      'status' => 'error',
    ];

    $data = Json::decode($request->getContent());
    if (empty($data['event_type'])) {
      $result['message'] = $this->t('Webhook payload does not contain the event type');
    }
    elseif ($data['event_type'] == 'issue') {
      $this->queueFactory->get('drupalorg_contribution_activity_webhook_queue_worker')->createItem([
        'event_name' => 'issue_update',
        'project_id' => $data['object_attributes']['project_id'] ?? NULL,
        'iid' => $data['object_attributes']['iid'] ?? NULL,
        'url' => $data['object_attributes']['url'],
        'action' => $data['object_attributes']['action'],
      ]);
      $result['message'] = $this->t('Event was queued successfully');
      $result['status'] = 'success';
    }
    elseif ($data['event_type'] == 'merge_request') {
      $this->queueFactory->get('drupalorg_contribution_activity_webhook_queue_worker')->createItem([
        'event_name' => 'merge_request_update',
        'project_id' => $data['project']['id'] ?? $data['object_attributes']['target_project_id'] ?? NULL,
        'iid' => $data['object_attributes']['iid'] ?? NULL,
        'url' => $data['object_attributes']['url'],
        'action' => $data['object_attributes']['action'],
      ]);
      $result['message'] = $this->t('Event was queued successfully');
      $result['status'] = 'success';
    }
    elseif ($data['event_type'] == 'note') {
      // For now, we only consider issues and merge requests.
      $type = !empty($data['issue']) ? 'issue' : 'merge_request';
      if (!empty($data[$type])) {
        $this->queueFactory->get('drupalorg_contribution_activity_webhook_queue_worker')->createItem([
          'event_name' => 'comment_update',
          'project_id' => $data['project_id'] ?? NULL,
          'iid' => $data[$type]['iid'] ?? NULL,
          'url' => $data[$type]['url'],
        ]);
        $result['message'] = $this->t('Event was queued successfully');
        $result['status'] = 'success';
      }
      else {
        $result['message'] = $this->t('The data provided did not have the expected format.');
      }
    }
    else {
      $result['message'] = $this->t('Event not covered by code');
      $result['status'] = 'warning';
    }

    return $result;
  }

  /**
   * Process the webhook request from Drupal.
   *
   * @param \Symfony\Component\HttpFoundation\Request $request
   *   Request object.
   *
   * @return array
   *   Result to return.
   */
  protected function contributionActivityWebhookFromDrupal(Request $request): array {
    $result = [
      'status' => 'error',
    ];

    $data = $request->request->all();
    if (empty($data['event_type'])) {
      $result['message'] = $this->t('Webhook payload does not contain the event type');
    }
    elseif ($data['event_type'] == 'issue') {
      $action = $data['action'] ?? '-';
      $this->queueFactory->get('drupalorg_contribution_activity_webhook_queue_worker')->createItem([
        'event_name' => 'drupalorg_issue_update',
        'url' => $data['url'],
        'action' => $action,
      ]);
      $result['message'] = $this->t('Event was queued successfully');
      $result['status'] = 'success';
    }
    elseif ($data['event_type'] == 'comment') {
      $this->queueFactory->get('drupalorg_contribution_activity_webhook_queue_worker')->createItem([
        'event_name' => 'drupalorg_comment_update',
        'url' => $data['url'],
      ]);
      $result['message'] = $this->t('Event was queued successfully');
      $result['status'] = 'success';
    }
    else {
      $result['message'] = $this->t('Event not covered by code');
      $result['status'] = 'warning';
    }

    return $result;
  }

}
