<?php

namespace Drupal\drupalorg\Plugin\QueueWorker;

use Drupal\contribution_records\SourceLink;
use Drupal\Core\Queue\QueueWorkerBase;
use Drupal\Core\StringTranslation\StringTranslationTrait;
use Drupal\drupalorg\Traits\GitLabClientTrait;

/**
 * Defines 'drupalorg_contribution_activity_webhook_queue_worker' queue worker.
 *
 * Run via `drush` like this:
 * `drush queue:run drupalorg_contribution_activity_webhook_queue_worker`.
 *
 * @QueueWorker(
 *   id = "drupalorg_contribution_activity_webhook_queue_worker",
 *   title = @Translation("Issue Activity Webhook Queue Worker")
 * )
 */
class DrupalOrgContributionActivityWebhookQueueWorker extends QueueWorkerBase {

  use StringTranslationTrait;
  use GitLabClientTrait;

  /**
   * {@inheritdoc}
   */
  public function processItem($data) {
    if (!$this->validateItem($data)) {
      return FALSE;
    }

    // All current webhooks events rely on this service, but the module is not
    // a hard dependency, so just check if it's available.
    if (!\Drupal::moduleHandler()->moduleExists('contribution_records')) {
      return FALSE;
    }

    switch ($data['event_name']) {
      case 'issue_update':
      case 'merge_request_update':
        $this->processContributionUpdate($data);
        break;

      case 'drupalorg_issue_update':
        $this->processDrupalOrgIssueUpdate($data);
        break;

      case 'comment_update':
      case 'drupalorg_comment_update':
        $this->processCommentUpdate($data);
        break;
    }
  }

  /**
   * Process the contribution update item from gitlab.
   *
   * @param array $data
   *   Data coming from the webhook.
   */
  protected function processContributionUpdate(array $data) {
    $source_link = $this->getSourceLink($data['url']);
    if ($source_link->isValid()) {
      $entity = $source_link->getLinkedEntity();

      $action = $data['action'] ?? '-';
      $markup = [];

      // Allow modules to add markup via hook alter.
      \Drupal::moduleHandler()->alter(
        'gitlab_contribution_automated_comment',
        $markup,
        $data
      );
      $markup = !empty($markup) ? implode('<hr>' . PHP_EOL, $markup) : '';

      // If the event is first creation, add the link to the description if
      // it's not there already.
      if ($action === 'open') {
        $this->addNote($markup, $data, $source_link);
      }
      // If the event is to close, merge or reopen the contribution, add a
      // comment with link and sync the contribution record.
      elseif (in_array($action, ['close', 'merge', 'reopen'])) {
        if ($entity) {
          $source_link->syncContribRecord($entity);
        }
        $this->addNote($markup, $data, $source_link);
      }
      // On contribution update, just sync contributors.
      elseif ($action === 'update') {
        if ($entity) {
          $source_link->addContributors($entity);
        }
        else {
          // Entity not created yet. Offer the links again that will create it.
          $this->addNote($markup, $data, $source_link);
        }
      }
    }
  }

  /**
   * Process the comment update item.
   *
   * @param array $data
   *   Data coming from the webhook.
   */
  protected function processCommentUpdate(array $data) {
    $source_link = $this->getSourceLink($data['url']);
    if ($source_link->isValid()) {
      // Sync the contributors on every comment.
      $entity = $source_link->getLinkedEntity();
      if ($entity) {
        $source_link->addContributors($entity);
      }
    }
  }

  /**
   * Validate item array to make sure all key elements are there.
   *
   * @param array $data
   *   Item to validate.
   *
   * @return bool
   *   Whether the item was valid or not.
   */
  protected function validateItem(array $data) {
    if (empty($data['event_name'])) {
      return FALSE;
    }

    $allowed_events = [
      'issue_update',
      'comment_update',
      'merge_request_update',
      'drupalorg_issue_update',
      'drupalorg_comment_update',
    ];
    if (!in_array($data['event_name'], $allowed_events)) {
      return FALSE;
    }

    // GitLab events.
    if (in_array($data['event_name'], [
      'issue_update',
      'comment_update',
      'merge_request_update',
    ])) {
      if (
        empty($data['iid']) ||
        empty($data['project_id']) ||
        empty($data['url'])
      ) {
        return FALSE;
      }
    }
    // Drupal.org events.
    elseif (in_array($data['event_name'], [
      'drupalorg_issue_update',
      'drupalorg_comment_update',
    ])) {
      if (
        empty($data['url'])
      ) {
        return FALSE;
      }
    }

    return TRUE;
  }

  /**
   * Process an issue update event from drupal.org.
   *
   * The logic to add first comment and last depending on the issue status
   * actually happens on D7 www.drupal.org itself, so we mostly update the
   * draft flag and sync contributors here.
   *
   * @param array $data
   *   Data from the webhook.
   */
  protected function processDrupalOrgIssueUpdate(array $data) {
    $source_link = $this->getSourceLink($data['url']);
    if ($source_link->isValid()) {
      $entity = $source_link->getLinkedEntity();
      if ($entity) {
        $action = $data['action'] ?? '-';
        if (in_array($action, ['close', 'update', 'reopen'])) {
          $source_link->syncContribRecord($entity);
        }
      }
      else {
        // No entity created yet, go ahead and create it then regardless of
        // the action. It will also sync contributors on save.
        $source_link->createContribRecord();
      }
    }
  }

  /**
   * Returns the source_link service already setup with the link in the array.
   *
   * @param string $url
   *   Url for the source link.
   *
   * @return \Drupal\contribution_records\SourceLink
   *   SourceLink object for the given link.
   */
  protected function getSourceLink(string $url) {
    /** @var \Drupal\contribution_records\SourceLink $source_link_instance */
    $source_link_instance = \Drupal::classResolver(SourceLink::class);
    return $source_link_instance->setLink($url);
  }

  /**
   * Adds a note to an issue from the given markup.
   *
   * @param string $markup
   *   Markup of the note.
   * @param array $data
   *   Array of data containing the project_id and iid.
   * @param \Drupal\contribution_records\SourceLink $source_link
   *   Source link object.
   */
  protected function addNote(string $markup, array $data, SourceLink $source_link) {
    if (!empty($markup)) {
      // For now, we only consider issues and merge requests.
      $method = ($source_link->getSourceType() === 'issue-gitlab') ? 'issues' : 'mergeRequests';
      $this->getGitLabClient()->{$method}()->addNote($data['project_id'], $data['iid'], $markup);
    }
  }

}
