<?php

namespace Drupal\drupalorg\Plugin\QueueWorker;

use Drupal\Core\Queue\QueueWorkerBase;
use Drupal\drupalorg\Traits\GitLabClientTrait;
use Drupal\user\Entity\User;

/**
 * Defines 'drupalorg_issue_forks_queue_worker' queue worker.
 *
 * Run via `drush` like this:
 * `drush queue:run drupalorg_issue_forks_queue_worker`.
 *
 * @QueueWorker(
 *   id = "drupalorg_issue_forks_queue_worker",
 *   title = @Translation("Issue Forks Queue Worker")
 * )
 */
class DrupalOrgIssueForksQueueWorker extends QueueWorkerBase {

  use GitLabClientTrait;

  /**
   * {@inheritdoc}
   */
  public function processItem($data) {
    if (!$this->validateItem($data)) {
      return FALSE;
    }

    switch ($data['action']) {
      case 'request_access':
        $this->requestAccess($data);
        break;

      case 'post_fork_creation':
        $this->postForkCreation($data);
        break;
    }
  }

  /**
   * Validate item array to make sure all key elements are there.
   *
   * @param array $data
   *   Item to validate.
   *
   * @return bool
   *   Whether the item was valid or not.
   */
  protected function validateItem(array $data) {
    if (
      empty($data['action']) ||
      empty($data['fork_id']) ||
      empty($data['issue_id']) ||
      empty($data['user_id'])
    ) {
      return FALSE;
    }

    return TRUE;
  }

  /**
   * Returns the gitlab user id for the given user if found.
   *
   * @param int $user_id
   *   Id of the user.
   *
   * @return int|null
   *   Id of the user in gitlab or null.
   */
  protected function getGitLabUserIdFromUserId(int $user_id) {
    $cache_id = 'gitlab_user_id_for_drupal_user_' . $user_id;
    if ($cached_data = \Drupal::cache()->get($cache_id)) {
      return $cached_data->data;
    }

    $user_gitlab_id = NULL;
    $client = $this->getGitLabClient();
    $drupal_user = User::load($user_id);
    $gitlab_username = $drupal_user->field_git_username->getValue()[0]['value'] ?? FALSE;
    if (empty($gitlab_username)) {
      return NULL;
    }
    else {
      try {
        $gitlab_users = $client->users()->all([
          'username' => $gitlab_username,
        ]);
        if (count($gitlab_users) == 1 && $gitlab_users[0]['username'] == $gitlab_username) {
          $user_gitlab_id = $gitlab_users[0]['id'];
          \Drupal::cache()->set($cache_id, $user_gitlab_id);
        }
      }
      catch (\Throwable $e) {
        \Drupal::logger('drupalorg')->error('Error searching users. Error: @error', [
          '@error' => $e->getMessage(),
        ]);
      }
    }

    return $user_gitlab_id;
  }

  /**
   * Gives the user access to the fork.
   *
   * @param array $data
   *   Data given to the queue containing user and fork information.
   */
  protected function requestAccess(array $data) {
    $client = $this->getGitLabClient();
    $gitlab_user_id = $this->getGitLabUserIdFromUserId($data['user_id']);
    if (!empty($gitlab_user_id)) {
      try {
        // 30 is “Developer access” https://docs.gitlab.com/ee/api/members.html
        $access_level = ($gitlab_user_id == 1) ? '50' : '30';
        $client->projects()->addMember($data['fork_id'], $gitlab_user_id, $access_level);
      }
      catch (\Throwable $e) {
        if ($e->getMessage() !== 'Member already exists') {
          \Drupal::logger('drupalorg')->error('Error adding member to fork. Message: @message', [
            '@message' => $e->getMessage(),
          ]);
        }
      }
    }
  }

  /**
   * Creates a branch based on the issue id and adds the user as member of the fork.
   *
   * @param array $data
   *   Data given to the queue containing user, issue and fork information.
   */
  protected function postForkCreation(array $data) {
    $client = $this->getGitLabClient();

    try {
      $fork = $client->projects()->show($data['fork_id']);

      // Create a branch with the issue number in it.
      if (!empty($fork['forked_from_project']['default_branch'])) {
        $branch_name = 'issue-' . $data['issue_id'] . '-branch';
        $client->repositories()->createBranch($data['fork_id'], $branch_name, $fork['forked_from_project']['default_branch']);
      }

      // Add the user that created the fork as member.
      $gitlab_user_id = $this->getGitLabUserIdFromUserId($data['user_id']);
      if ($gitlab_user_id) {
        $access_level = ($gitlab_user_id == 1) ? '50' : '30';
        $client->projects()->addMember($fork['id'], $gitlab_user_id, $access_level);
      }
    }
    catch (\Throwable $e) {
      \Drupal::logger('drupalorg')->error('Error in post fork creation. Message: @message', [
        '@message' => $e->getMessage(),
      ]);
    }
  }

}
