<?php

namespace Drupal\drupalorg\Plugin\QueueWorker;

use Drupal\Core\Queue\QueueWorkerBase;
use Drupal\Core\StringTranslation\StringTranslationTrait;
use Drupal\drupalorg\Traits\GitLabClientTrait;

/**
 * Defines 'drupalorg_project_activity_webhook_queue_worker' queue worker.
 *
 * Run via `drush` like this:
 * `drush queue:run drupalorg_project_activity_webhook_queue_worker`.
 *
 * @QueueWorker(
 *   id = "drupalorg_project_activity_webhook_queue_worker",
 *   title = @Translation("Project Activity Webhook Queue Worker")
 * )
 */
class DrupalOrgProjectActivityWebhookQueueWorker extends QueueWorkerBase {

  use StringTranslationTrait;
  use GitLabClientTrait;

  /**
   * {@inheritdoc}
   */
  public function processItem($data) {
    if (!$this->validateItem($data)) {
      return FALSE;
    }

    if ($data['event_name'] == 'project_update') {
      $this->processProjectUpdate($data);
    }
  }

  /**
   * Process the project update item.
   *
   * @param array $data
   *   Data coming from the webhook.
   */
  protected function processProjectUpdate(array $data) {
    $gitlab_client = $this->getGitLabClient();
    $project = $gitlab_client->projects()->show($data['project_id']);
    if ($project['avatar_url']) {
      /** @var \Drupal\drupalorg\ProjectService $project_service */
      $project_service = \Drupal::service('drupalorg.project_service');
      $drupal_project = $project_service->getProjectByComposerNamespace('drupal/' . $project['name']);
      if ($drupal_project) {
        if (empty($drupal_project->get('field_logo_url')->getValue())) {
          $drupal_project->set('field_logo_url', $project['avatar_url'])->save();
        }
      }
    }
  }

  /**
   * Validate item array to make sure all key elements are there.
   *
   * @param array $data
   *   Item to validate.
   *
   * @return bool
   *   Whether the item was valid or not.
   */
  protected function validateItem(array $data) {
    if (empty($data['event_name'])) {
      return FALSE;
    }

    $allowed_events = [
      'project_update',
    ];
    if (!in_array($data['event_name'], $allowed_events)) {
      return FALSE;
    }

    if ($data['event_name'] == 'project_update') {
      if (empty($data['project_id'])) {
        return FALSE;
      }
    }

    return TRUE;
  }

}
