<?php

namespace Drupal\drupalorg;

use Drupal\Core\Database\Connection;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\node\NodeInterface;

/**
 * Project related helper methods.
 */
class ProjectService {

  /**
   * The database connection.
   *
   * @var \Drupal\Core\Database\Connection
   */
  protected Connection $connection;

  /**
   * An instance of the entity type manager.
   *
   * @var \Drupal\Core\Entity\EntityTypeManagerInterface
   */
  protected $entityTypeManager;

  /**
   * Construct method.
   *
   * @param \Drupal\Core\Database\Connection $connection
   *   The database connection.
   * @param \Drupal\Core\Entity\EntityTypeManagerInterface $entityTypeManager
   *   Entity type manager service.
   */
  public function __construct(Connection $connection, EntityTypeManagerInterface $entityTypeManager) {
    $this->connection = $connection;
    $this->entityTypeManager = $entityTypeManager;
  }

  /**
   * Retrieve a project given a value and a field.
   *
   * @param string $field
   *   Field to query.
   * @param string $value
   *   Value to check.
   *
   * @return \Drupal\node\NodeInterface|null
   *   Project node or null.
   */
  protected function getProjectBy(string $field, string $value): ?NodeInterface {
    $project_id = $this->entityTypeManager->getStorage('node')->getQuery()
      ->accessCheck(FALSE)
      ->condition('type', 'project_module')
      ->condition('status', NodeInterface::PUBLISHED)
      ->condition($field, $value)
      ->range(0, 1)
      ->execute();
    if (!empty($project_id)) {
      return $this->entityTypeManager->getStorage('node')->load(reset($project_id));
    }

    return NULL;
  }

  /**
   * Retrieve a project from its composer username.
   *
   * @param string $composer_namespace
   *   Composer namespace of the project.
   *
   * @return \Drupal\node\NodeInterface|null
   *   Project node or null.
   */
  public function getProjectByComposerNamespace(string $composer_namespace): ?NodeInterface {
    return $this->getProjectBy('field_composer_namespace', $composer_namespace);
  }

  /**
   * Retrieve a project from its machine name.
   *
   * @param string $machine_name
   *   Machine name of the project.
   *
   * @return \Drupal\node\NodeInterface|null
   *   Project node or null.
   */
  public function getProjectByMachineName(string $machine_name): ?NodeInterface {
    return $this->getProjectBy('field_project_machine_name', $machine_name);
  }

}
