<?php

namespace Drupal\drupalorg;

use Drupal\Core\Database\Connection;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\user\UserInterface;

/**
 * User related helper methods.
 */
class UserService {

  /**
   * The database connection.
   *
   * @var \Drupal\Core\Database\Connection
   */
  protected Connection $connection;

  /**
   * An instance of the entity type manager.
   *
   * @var \Drupal\Core\Entity\EntityTypeManagerInterface
   */
  protected $entityTypeManager;

  /**
   * Construct method.
   *
   * @param \Drupal\Core\Database\Connection $connection
   *   The database connection.
   * @param \Drupal\Core\Entity\EntityTypeManagerInterface $entityTypeManager
   *   Entity type manager service.
   */
  public function __construct(Connection $connection, EntityTypeManagerInterface $entityTypeManager) {
    $this->connection = $connection;
    $this->entityTypeManager = $entityTypeManager;
  }

  /**
   * Retrieve user by username.
   *
   * @param string $username
   *   Drupal username.
   * @param bool $ensure_format
   *   Make sure that the username format is the expected.
   *
   * @return \Drupal\user\UserInterface|null
   *   User entity or null.
   */
  public function getUserByUsername(string $username, bool $ensure_format = TRUE): ?UserInterface {
    $users = $this->getUsersByUsername([$username], $ensure_format);
    return !empty($users) ? array_shift($users) : NULL;
  }

  /**
   * Retrieve users by name.
   *
   * @param string[] $usernames
   *   List of usernames.
   * @param bool $ensure_format
   *   Make sure that the username format is the expected.
   *
   * @return \Drupal\user\UserInterface[]|null
   *   User entity or null.
   */
  public function getUsersByUsername(array $usernames, bool $ensure_format = TRUE): ?array {
    if ($ensure_format) {
      $usernames = $this->ensureUsernameFormat($usernames);
    }
    return $this->getUsersBy('name', $usernames);
  }

  /**
   * Transform D7 usernames into D9 format usernames.
   *
   * Usernames were "massaged" during the migration from D7 to D9, so they
   * might differ. Apply the same transformations that were applied.
   *
   * @param array $usernames
   *   D7 usernames.
   *
   * @return array
   *   D9 usernames.
   */
  protected function ensureUsernameFormat(array $usernames): array {
    $transformed = [];
    foreach ($usernames as $username) {
      // Machine name transformation.
      $new_value = strtolower($username);
      $new_value = preg_replace('/[^a-z0-9_]+/', '_', $new_value);
      $new_value = preg_replace('/_+/', '_', $new_value);

      // Substr transformation.
      $new_value = substr($new_value, 0, 60);

      $transformed[] = $new_value;
    }

    return $transformed;
  }

  /**
   * Retrieve user by git username.
   *
   * @param string $username
   *   Git username.
   *
   * @return \Drupal\user\UserInterface|null
   *   User entity or null.
   */
  public function getUserByGitUsername(string $username): ?UserInterface {
    $users = $this->getUsersByGitUsername([$username]);
    return !empty($users) ? array_shift($users) : NULL;
  }

  /**
   * Retrieve users by git username.
   *
   * @param string[] $usernames
   *   List of git usernames.
   *
   * @return \Drupal\user\UserInterface[]|null
   *   User entity or null.
   */
  public function getUsersByGitUsername(array $usernames): ?array {
    return $this->getUsersBy('field_git_username', $usernames);
  }

  /**
   * Generic function to get users by a certain value on a given field.
   *
   * @param string $field
   *   Name of the field.
   * @param array $values
   *   List of values for the field.
   *
   * @return \Drupal\user\UserInterface[]|null
   *   List of users matching the criteria or null.
   */
  protected function getUsersBy($field, array $values): ?array {
    $ids = $this->entityTypeManager->getStorage('user')->getQuery()
      ->accessCheck(FALSE)
      ->condition($field, $values, 'IN')
      ->condition('status', 1)
      ->execute();
    if (!empty($ids)) {
      return $this->entityTypeManager->getStorage('user')->loadMultiple($ids);
    }

    return NULL;
  }

}
